/*****************************************************************************
 * © 2014 Avaya Inc. All rights reserved.
 ****************************************************************************/
package com.avaya.services.Callingpolicies;

import com.avaya.collaboration.call.Call;

interface PromptAndCollectOperation
{
    void executePromptAndCollectOperation(Call call);

    void cancelSequentialForkingTimer(Call call);

    void startNoAnswerTimer(Call call);

    void addPartyToCall(Call call);
}
